
#include <GL/glut.h>

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <limits.h>           /* ULONG_MAX is defined here */
#include <float.h>            /* FLT_MAX is atleast defined here */

#include <time.h>  /* for random seed */

//#include "fracviewer.h"

#ifdef _WIN32
#define drand48() (((float) rand())/((float) RAND_MAX))
#define srand48(x) (srand((x)))
#endif

typedef struct { float x,y,z;  }point;

typedef enum { NOTALLOWED, MOUNTAIN, BIGMTN, MOUNTAIN_MAT} DisplayLists;

#define MAXLEVEL 6

int tempo;
/***************************************************************/
/************************* VECTOR JUNK *************************/
/***************************************************************/

  
  /* normalizes v */
void normalize(point*   v   )
{
  GLfloat d = sqrt(v->x *v->x  + v->y *v->y  + v->z *v->z );

  if (d == 0)
    fprintf(stderr, "Zero length vector in normalize\n");
  else
    v->x  /= d; v->y  /= d; v->z  /= d;
}

  /* calculates a normalized crossproduct to v1, v2 */
void ncrossprod(point v1, point v2,point* cp)
{
  cp->x  = v1.y *v2.z  - v1.z *v2.y ;
  cp->y  = v1.z *v2.x  - v1.x *v2.z ;
  cp->z  = v1.x *v2.y  - v1.y *v2.x ;
  normalize(cp);
}

  /* calculates normal to the triangle designated by v1, v2, v3 */
void triagnormal(point v1, point v2, point v3, point* norm)
{
  point vec1, vec2;

  vec1.x = v3.x - v1.x;  vec2.x = v2.x - v1.x;
  vec1.y = v3.y - v1.y;  vec2.y = v2.y - v1.y;
  vec1.z = v3.z - v1.z;  vec2.z = v2.z - v1.z;

  ncrossprod(vec2, vec1, norm);
}



/***************************************************************/
/************************ MOUNTAIN STUFF ***********************/
/***************************************************************/

        
float Segno(int n);



void myGLInit(void)
{
  GLfloat light_ambient[] = { 0.0, 0.0, 0.0, 1.0 };
  GLfloat light_diffuse[] = { 1.0, 1.0, 1.0, 1.0 };
  GLfloat light_specular[] = { 1.0, 1.0, 1.0, 1.0 };
  GLfloat light_position[] = { 0, 0.5, 0.1, 0.0 };

  GLfloat lmodel_ambient[] = { 0.4, 0.4, 0.4, 1.0 };

  glLightfv(GL_LIGHT0, GL_AMBIENT, light_ambient);
  glLightfv(GL_LIGHT0, GL_DIFFUSE, light_diffuse);
  glLightfv(GL_LIGHT0, GL_SPECULAR, light_specular);
  glLightfv(GL_LIGHT0, GL_POSITION, light_position);
    
  glLightModelfv(GL_LIGHT_MODEL_AMBIENT, lmodel_ambient);
  
  glEnable(GL_LIGHTING);
  glEnable(GL_LIGHT0);
  
  glDepthFunc(GL_LEQUAL);
  glEnable(GL_DEPTH_TEST);

  glEnable(GL_NORMALIZE);

  glShadeModel(GL_SMOOTH);
  
  glClearColor (1, 1, 1.2, 0.0);

  glFlush();
} 

/***************************************************************/
/************************ GLUT STUFF ***************************/
/***************************************************************/
static void  Key(unsigned char key, int x, int y)
{
  switch (key) {
  case 27:
    exit(0);
  }
}


float Segno(int n) {
  return ((n%2)==0) ? 1. : -1.;
}

void MidPoint(point* c,point a, point b,int level){
  int num;

  num=(int)((a.x*b.x)*560+(a.y+b.y)*263+(a.z*b.z)*766);
  srand48(num+tempo);
  c->x=(a.x+b.x)/2.;
  c->y=(a.y+b.y)/2.+0.5*((0.5-drand48())*sqrt((a.x-b.x)*(a.x-b.x)+(a.y-b.y)*(a.y-b.y)+(a.z-b.z)*(a.z-b.z))) ;
  c->z=(a.z+b.z)/2.;

}

void DrawFractalMount(int level, point a, point b, point c){
  point m1,m2,m3,norm;
  GLfloat color[4]={ 1.3,0.6,0.6,1};
 
  if (level!=0){
    MidPoint(&m1,a,b,level);
    MidPoint(&m2,b,c,level);
    MidPoint(&m3,c,a,level);

    DrawFractalMount(level-1,a,m1,m3);
    DrawFractalMount(level-1,m1,b,m2);
    DrawFractalMount(level-1,m2,m3,m1);
    DrawFractalMount(level-1,m2,c,m3);

  }
  else{
    
    triagnormal(a,b,c,&norm);
    
    glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, color);
     
    glBegin(GL_TRIANGLE_STRIP);
    glNormal3f(norm.x,norm.y,norm.z);    
    glVertex3f(a.x,a.y,a.z);
    glVertex3f(b.x,b.y,b.z);
    glVertex3f(c.x,c.y,c.z);
    glEnd();

  }
}


/*Perche' disegna soltanto meta' del rettangolo del lago??????.
Inoltre mi comunica il messaggio 
"First-chance exception in fractal.exe .......:Access Violation"
alla fine dell'esuzione . Cosa significa? */

void DrawWater(float level){
 GLfloat color[4]={ 0.3,0.5,0.9,1};
 
	glNormal3f(0,1,0);
		
   	glBegin(GL_TRIANGLE_STRIP);

    glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, color);
	glVertex3f(18,level,18);
    glVertex3f(-18,level,18);
    glVertex3f(18,level,-18);
    glVertex3f(-18, level,-18);    
    
	glEnd();

}


void display(void)
{ 
  point a={20,5,20}, b={-20,5,20},c={20,5,-20},d={-20,5,-20};
  static angle=0;
  
  

  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  glFlush();

  glMatrixMode(GL_PROJECTION);
  glPopMatrix();
  glPushMatrix();  /* clear of last viewing xform, leaving perspective */

    glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  
  glRotatef(angle,0,1,0);

/*le due parti della montagna sono molto diverse tra loro, una e' piu' "aguzza" dell'altra.
  Perche'????*/
  DrawFractalMount(MAXLEVEL,a,c,b);
  DrawFractalMount(MAXLEVEL,c,d,b);
	
  DrawWater(9.);

  glutSwapBuffers();

  angle=(3+angle)%360;
}


void reshape(GLsizei w, GLsizei h)
{
  glViewport(0,0,w,h);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluPerspective(60.0, (GLdouble)w/h, 0.01, 100);
  gluLookAt(40,40,40,
		0,0,0,
		0,1,0);
  glPushMatrix();
  glMatrixMode(GL_MODELVIEW);
  glFlush();

}


/***************************************************************/
/******************* MENU SETUP & HANDLING *********************/
/***************************************************************/

typedef enum { MENU_QUIT, MENU_RAND, MENU_MOVE, MENU_AXES } MenuChoices;

void setlevel(int value)
{
  //Level = value;
  //Rebuild = 1;
  glutPostRedisplay();
}



/***************************************************************/
/**************************** MAIN *****************************/
/***************************************************************/

int main(int argc, char** argv)
{
  tempo=time(0);

  glutInit(&argc, argv);
  glutInitWindowSize(512, 512);
  glutInitWindowPosition (10, 10);  
  glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGBA | GLUT_DEPTH );
  glutCreateWindow("Fractal Planet");

  glutReshapeFunc(reshape);
  glutDisplayFunc(display);
  glutKeyboardFunc(Key);	
  glutIdleFunc(display);
  
  myGLInit(); 
  
  glutMainLoop();
  return 0;             /* ANSI C requires main to return int. */
}









